@@ -1,7 +1,7 @@
 #ifndef VOR_CONFIG_H
 #define VOR_CONFIG_H
 
-#define VERSION "0.5.3"
+#define VERSION "0.5.4"
 
 // screen size
 #define XSIZE 640
@@ -57,4 +57,7 @@
 
 #define MAX_PATH_LEN 1024
 
+// number of milliseconds the blinky cursor stays visible/invisible
+#define CURSOR_BLINK_TIME 300
+
 #endif // VOR_CONFIG_H
