@@ -68,12 +68,15 @@ struct dot {
 	int heat;     // heat multiplier (color).
 };
 
+void draw(void);
+
 struct dot edot[MAXENGINEDOTS], *dotptr = edot;
 struct dot bdot[MAXBANGDOTS];
 
 // Other global variables
 char topline[1024];
 char *initerror = "";
+int screenshot_number = 0;
 
 struct ship ship = { SHIP, 0, NULL, XSIZE/2, YSIZE/2, BARRIER_SPEED, 0.0 };
 	  
@@ -92,7 +95,7 @@ int score;
 
 float fadetimer = 0;
 
-int pausedown = 0, paused = 0;
+int paused = 0;
 
 // bangdot start (bd1) and end (bd2) position:
 int bd1 = 0, bd2 = 0;
@@ -347,12 +350,31 @@ void font_cleanup() {
 	font_free(g_font);
 }
 
+void
+set_video_mode() {
+	Uint32 flag;
+
+	// Attempt to get the required video size
+	flag = SDL_DOUBLEBUF | SDL_HWSURFACE;
+	if(opt_fullscreen) flag |= SDL_FULLSCREEN;
+	surf_screen = SDL_SetVideoMode(XSIZE,YSIZE,16,flag);
+}
+
+void
+toggle_fullscreen() {
+	opt_fullscreen = 1 - opt_fullscreen;
+	set_video_mode();
+	if(paused) {
+		draw();
+	}
+}
+
+
 int
 init(void) {
 
 	int i;
 	char *s;
-	Uint32 flag;
 
 	// Where are our data files?
 	if(!find_files()) exit(1);
@@ -377,10 +399,9 @@ init(void) {
 
 	play_tune(TUNE_TITLE_PAGE);
 
+
 	// Attempt to get the required video size
-	flag = SDL_DOUBLEBUF | SDL_HWSURFACE;
-	if(opt_fullscreen) flag |= SDL_FULLSCREEN;
-	surf_screen = SDL_SetVideoMode(XSIZE,YSIZE,16,flag);
+	set_video_mode();
 
 	// Set the title bar text
 	SDL_WM_SetCaption("Variations on Rockdodger", "VoR");
@@ -517,16 +538,19 @@ draw(void)
 	show_score();
 
 	switch (state) {
-		case GAME_OVER: draw_game_over(); break;
+		case GAME_OVER:
+			draw_game_over();
+			break;
 
-		case TITLE_PAGE: draw_title_page(); break;
+		case TITLE_PAGE:
+			draw_title_page();
+			break;
 
-		case HIGH_SCORE_ENTRY: play_tune(TUNE_HIGH_SCORE_ENTRY);
-			// and fall through to
+		case HIGH_SCORE_ENTRY:
 		case HIGH_SCORE_DISPLAY:
-			// Display de list o high scores mon.
 			display_scores(150,50);
 			break;
+
 		case GAMEPLAY:
 		case DEAD_PAUSE:
 			; // no action necessary
@@ -552,6 +576,7 @@ kill_ship(struct ship *ship)
 		if(ship->dx < BARRIER_SPEED) ship->dx = BARRIER_SPEED;
 	} else {
 		state = GAME_OVER;
+		play_tune(TUNE_TITLE_PAGE);
 		state_timeout = 200.0;
 		fadetimer = 0.0;
 		ship->flags = 0;
@@ -593,7 +618,6 @@ update_state(void)
 			// Restore the ship and continue playing
 			ship.flags = DRAW|MOVE|COLLIDE;
 			state = GAMEPLAY;
-			play_tune(TUNE_GAMEPLAY);
 			break;
 		case GAME_OVER:
 			if(new_high_score(score)) init_score_entry();
@@ -635,10 +659,15 @@ gameloop() {
 			autopilot(t_frame);
 		}
 
-		while(SDL_PollEvent(&e)) {
+		while(paused ? SDL_WaitEvent(&e) : SDL_PollEvent(&e)) {
 			switch(e.type) {
 				case SDL_QUIT: return;
 				case SDL_KEYDOWN:
+					// even during high-score entry
+					if(e.key.keysym.sym == SDLK_ESCAPE) {
+						return;
+					}
+
 					if(state == HIGH_SCORE_ENTRY) {
 						if(!process_score_input(&e.key.keysym)) {
 							// Write the high score table to the file
@@ -646,14 +675,47 @@ gameloop() {
 							// continue to display the scores briefly
 							state = HIGH_SCORE_DISPLAY;
 							state_timeout = 200;
-							play_tune(TUNE_TITLE_PAGE);
 						}
-					} else if(e.key.keysym.sym == SDLK_q) {
-						return;
-					}
-
-					if(e.key.keysym.sym == SDLK_ESCAPE) {
-						return;
+					} else {
+						switch(e.key.keysym.sym) {
+							case SDLK_q:
+								return;
+							case SDLK_3:
+							case SDLK_PRINT:
+								{
+									FILE *screenshot_fp;
+									char tmp[30];
+									char *screenshot_filename = &(tmp[0]);
+									for(;;) {
+										snprintf(screenshot_filename, 30, "vor-screenshot-%02i.bmp", screenshot_number++);
+										screenshot_fp = fopen(screenshot_filename, "r");
+										if(screenshot_fp) {
+											fclose(screenshot_fp);
+										} else {
+											break;
+										}
+									}
+									SDL_SaveBMP(surf_screen, screenshot_filename);
+								}
+								break;
+							case SDLK_p:
+							case SDLK_PAUSE:
+								paused = !paused;
+								if(paused) {
+									pause_tune();
+								} else {
+									resume_tune();
+									ms_end = SDL_GetTicks();
+								}
+								break;
+							case SDLK_f:
+							case SDLK_F11:
+								toggle_fullscreen();
+								break;
+							default:
+								// other keys are handled by checking keystate each frame
+								break;
+						}
 					}
 					break;
 			}
@@ -667,26 +729,24 @@ gameloop() {
 			if(!paused) {
 				score += ms_frame;
 				
-				if(keystate[SDLK_LEFT]  || keystate[SDLK_h]) { ship.dx -= THRUSTER_STRENGTH*t_frame; ship.jets |= 1<<0;}
-				if(keystate[SDLK_DOWN]  || keystate[SDLK_t]) { ship.dy += THRUSTER_STRENGTH*t_frame; ship.jets |= 1<<1;}
-				if(keystate[SDLK_RIGHT] || keystate[SDLK_n]) { ship.dx += THRUSTER_STRENGTH*t_frame; ship.jets |= 1<<2;}
-				if(keystate[SDLK_UP]    || keystate[SDLK_c]) { ship.dy -= THRUSTER_STRENGTH*t_frame; ship.jets |= 1<<3;}
+				if(keystate[SDLK_LEFT]  || keystate[SDLK_KP4]) {
+					ship.dx -= THRUSTER_STRENGTH*t_frame; ship.jets |= 1<<0;
+				}
+				if(keystate[SDLK_DOWN]  || keystate[SDLK_KP5] || keystate[SDLK_KP2]) {
+					ship.dy += THRUSTER_STRENGTH*t_frame; ship.jets |= 1<<1;
+				}
+				if(keystate[SDLK_RIGHT] || keystate[SDLK_KP6]) {
+					ship.dx += THRUSTER_STRENGTH*t_frame; ship.jets |= 1<<2;
+				}
+				if(keystate[SDLK_UP]    || keystate[SDLK_KP8]) {
+					ship.dy -= THRUSTER_STRENGTH*t_frame; ship.jets |= 1<<3;
+				}
 				if(ship.jets) {
 					ship.dx = fconstrain2(ship.dx, -50, 50);
 					ship.dy = fconstrain2(ship.dy, -50, 50);
 				}
-				if(keystate[SDLK_3])		{ SDL_SaveBMP(surf_screen, "snapshot.bmp"); }
 			}
 
-			if(keystate[SDLK_p] | keystate[SDLK_s]) {
-				if(!pausedown) {
-					paused = !paused;
-					pausedown = 1;
-					if(!paused) ms_end = SDL_GetTicks();
-				}
-			} else {
-				pausedown = 0;
-			}
 		}
 
 		if(!paused) {
